@@ -31,6 +31,33 @@
 
 extern int execute_instruction(harglst *, char *);
 
+
+static
+int balanced_starts_and_ends(buf)
+ char * buf;
+{
+ int count = 0;
+ while(buf[0])
+ {
+  if(buf[0] == START_BLOCK)count++;
+  else if(buf[0] == END_BLOCK)count --;
+  if(buf[0] == OPEN_QUOTE)
+  	{
+  	buf = strchr(buf+1, CLOSE_QUOTE);
+	if(!buf)break;
+	}
+  if(count < 0)return -1;
+  buf++;
+ }
+ if(count)
+  return 1;
+ else
+  return 0;
+  
+}
+
+
+
 int
 execute_script(globals, name)
  harglst * globals;
@@ -42,8 +69,14 @@ execute_script(globals, name)
  struct stat sb;
  int len;
  int err;
+ 
+ if(harg_get_string(globals,"script_name"))
+  harg_set_string(globals, "script_name", name);
+ else
+  harg_add_string(globals, "script_name", name);
+  
  if(fd < 0){
-  perror("open ");
+  perror("nasl: open() ");
   return(-1);
   }
  fstat(fd, &sb);
@@ -51,12 +84,31 @@ execute_script(globals, name)
  command = mmap(NULL, len, PROT_READ, MAP_PRIVATE, fd, 0);
  if(command == MAP_FAILED)
  {
-  perror("mmap ");
+  perror("nasl: mmap() ");
   return(-1);
  }
  buffer = nasl_strdup(globals,command);
- munmap(command, len);
- close(fd);
+ if(munmap(command, len))
+ {
+  perror("nasl : munmap() ");
+  return(-1);
+  }
+ if(close(fd))
+ {
+  perror("nasl : close() ");
+  return -1;
+ }
+ 
+ if(balanced_starts_and_ends(buffer))
+ {
+  fprintf(stderr, "%s - Parse error : unbalanced number of %c and %c\n", 
+  				harg_get_string(globals, "script_name"),
+  				START_BLOCK,
+				END_BLOCK);
+  return PKT_ERROR(ERR_PARSE);
+ }	
+ 
+ 
  err = execute_script_buffer(globals, buffer);
  nasl_free(globals, buffer);
  return(err);
@@ -68,14 +120,16 @@ int execute_script_buffer(globals, buffer)
 {
   char * r = NULL;
   char * inst;
-  char * buf = buffer; 
+  char * buf = buffer;
   int err = 0;
   while((inst = read_buf_instruction(globals, buf, &r)))
   {
    char * t = (char*)remove_whitespaces(globals, inst);
+   
    if(!t)return PKT_ERROR(ERR_PARSE);
    nasl_free(globals, inst);
    err = execute_instruction(globals, t);
+   nasl_free(globals, t);
    if(err<0)break;
    buf = r;
   }
