@@ -1,6 +1,6 @@
 /* Nessus Attack Scripting Language
  *
- * Copyright (C) 1999 Renaud Deraison
+ * Copyright (C) 1999, 2000 Renaud Deraison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,15 +30,17 @@
 #include "instruction.h"
 #include "execute.h"
 
+
+ 
 /*
  * Input : 'function(a:1, b, c:3)'
  *
  * Output : 
- * 	- function
- *	- args :
- *		'a' : 1
- *		''  : b
- *		'c' : 3
+ *      - function
+ *      - args :
+ *              'a' : 1
+ *              ''  : b
+ *              'c' : 3
  *
  */
 static harglst * split_function_args(globals, args)
@@ -46,11 +48,11 @@ static harglst * split_function_args(globals, args)
  char * args;
 {
  harglst * ret = harg_create(10);
-			/* The arglists use emalloc() for creating
-			   all the 'nexts'. So the head is not
-			   created with nasl_malloc()
-			 */
- struct arglist * fargs = emalloc(sizeof(struct arglist));
+                        /* The arglists use emalloc() for creating
+                           all the 'nexts'. So the head is not
+                           created with nasl_malloc()
+                         */
+ struct arglist * fargs = nasl_malloc(globals, sizeof(struct arglist));
  char * copy; 
  char * v, *e;
  char * end;
@@ -61,9 +63,10 @@ static harglst * split_function_args(globals, args)
  v = strchr(copy, START_FUNCTION);
  e = my_strchr(copy, START_FUNCTION, END_FUNCTION);
  if(!e){
- 	printf("ERROR : no %c in  %s\n", END_FUNCTION, copy);
-	return(NULL);
-	}
+        fprintf(stderr, "Parse error : no %c in  %s\n", END_FUNCTION, copy);
+        nasl_free(globals, copy);
+        return(NULL);
+        }
  v[0] = '\0';v++;
  e[0] = '\0';
  
@@ -74,7 +77,7 @@ static harglst * split_function_args(globals, args)
   */
  while(v && v[0])
  {
-  char * w;
+  char * w = NULL;
   e = v;
   /*
    * We look for the arguments separator, taking care
@@ -82,36 +85,36 @@ static harglst * split_function_args(globals, args)
    */
   while(e && e[0] && e[0]!=FUNCTION_ARGS_SEP){
 #ifdef DEBUG
-	printf("%s - e[0] : %c\n", e, e[0]);
-#endif	
-  	if(e[0]==OPEN_QUOTE)e = strchr(e+1, CLOSE_QUOTE);
-	else if(e[0]==START_FUNCTION)e=strchr(e+1, END_FUNCTION);
-	if(e && e[0])e++;
-	}
-	
+        printf("%s - e[0] : %c\n", e, e[0]);
+#endif  
+        if(e[0]==OPEN_QUOTE)e = strchr(e+1, CLOSE_QUOTE);
+        else if(e[0]==START_FUNCTION)e=my_strchr(e, START_FUNCTION, END_FUNCTION);
+        if(e && e[0])e++;
+        }
+        
  
   if(e && e[0])e[0]='\0';
   
   /*
    * v now points on xxxxxxx:xxxxxx or xxxxxx
    */
+   
+#ifdef DEBUG   
+   printf("V : %s\n", v);
+#endif   
    w = quoted_parenthesed_strchr(v, FUNCTION_ARGS);
    
    if(w&&!is_function(v))
    {
     /* Nominated arguments */
-    w[0] = '\0';
-#ifdef DEBUG
-    printf("add : %s:%s\n", v, w+1);
-#endif    
-    arg_add_value(fargs, v, ARG_STRING, strlen(w+1), nasl_strdup(globals,w+1));
+    w[0] = '\0'; 
+    nasl_arg_add_value(globals, fargs, v, VAR_STR,  strlen(w+1), w+1);   
    }
-   else
+   else /* anonymous arguments */
    {
-   arg_add_value(fargs, "no_name", ARG_STRING, strlen(v), nasl_strdup(globals,v));
-#ifdef DEBUG   
-   printf("add : %s\n", v);
-#endif   
+    if(v){
+    	nasl_arg_add_value(globals, fargs, "no_name", VAR_STR,  strlen(v), v); 
+	} 
    }
  if(e)v = e+1;  
  else v = NULL;
@@ -146,7 +149,7 @@ function_call_user_save_args(globals, args)
   harglst * args;
 {
  harglst * variables = harg_get_harg(globals, "variables");
- struct arglist * ret = emalloc(sizeof(struct arglist));
+ struct arglist * ret = nasl_malloc(globals, sizeof(struct arglist));
  hargwalk * hw = harg_walk_init(args);
  const char * key;
  
@@ -166,13 +169,15 @@ function_call_user_save_args(globals, args)
    {
    char * buf = nasl_malloc(globals, rt.length);
    memcpy(buf, rt.value, rt.length); 
-   arg_add_value(ret, name, rt.type, rt.length, buf);
+   nasl_arg_add_value(globals, ret, name, rt.type, rt.length, buf);
    }
    else
-    arg_add_value(ret,name, rt.type, rt.length, rt.value);
+    nasl_arg_add_value(globals, ret,name, rt.type, rt.length, rt.value);
   }
+  if(rt.type & VAR_DELETE)nasl_free(globals, rt.value);
   }
  }
+ harg_walk_stop(hw);
 return ret;
 }
 
@@ -193,6 +198,7 @@ function_call_user_apply_args(globals, fargs, args)
    {
    sane = sanitize_variable(globals, args->value);
    affect_var(globals, sane, name);
+   if(sane.type & VAR_DELETE)nasl_free(globals, sane.value);
    }
    args = args->next;
    
@@ -210,11 +216,12 @@ function_call_user_restore_args(globals, args)
   struct arglist * next = args->next;
   
   affect_var(globals, *args, args->name);
-  free(args->name);
-  free(args);
+  nasl_free(globals, args->name);
+  nasl_free(globals, args->value);
+  nasl_free(globals, args);
   args = next;
  }
- if(args)free(args);
+ if(args)nasl_free(globals, args);
 }
 static struct arglist
 function_call_user(globals, fdata)
@@ -229,7 +236,10 @@ function_call_user(globals, fdata)
   struct arglist nothing;
   int err;
    
+  
   bzero(&rt, sizeof(rt));
+  
+  
   if(!func)
   {
     rt.type = -1;
@@ -239,17 +249,19 @@ function_call_user(globals, fdata)
   /*
    * First, reset the return value
    */
-  nothing.value = strdup("0");
+  nothing.value = nasl_strdup(globals, "0");
   nothing.length = 1;
   nothing.type = VAR_STR|STR_ALL_DIGIT;
   affect_var(globals, nothing, "__nasl_return");
-
+  nasl_free(globals, nothing.value);
+  
   fargs = harg_get_harg(func,"arguments");
   /*
    * Now save the arguements
    */
-  old_args = function_call_user_save_args(globals,fargs);
-
+ 
+ old_args = function_call_user_save_args(globals,fargs);
+ 
   /* 
    * Apply the new args
    */
@@ -258,19 +270,23 @@ function_call_user(globals, fdata)
   /*
    * Execute the function in this new environement
    */
+   
   err = execute_script_buffer(globals, harg_get_string(func, "body")); 
+ 
   if((err < 0) && (err != PKT_ERROR(ERR_FUNC_EXIT)))
   {
    bzero(&nothing, sizeof(nothing));
    nothing.type = PKT_ERROR(ERR_EXIT);
    return nothing;
   }
+
   /*
    * Restores the arguments  
    */
+  
   function_call_user_restore_args(globals, old_args);
   return sanitize_variable(globals, "__nasl_return"); 
-
+ 
 }
 
 
@@ -279,40 +295,46 @@ function_call(globals, args)
  harglst * globals;
  char * args;
 {
- harglst * fdata 		= split_function_args(globals, args);
- harglst * functions 		= harg_get_harg(globals, "functions");
+ 
+ harglst * fdata                = split_function_args(globals, args);
+ harglst * functions            = harg_get_harg(globals, "functions");
  callback_func_t func;
  struct arglist * arguments;
+ struct arglist * delete_us;
  int repeat = 1;
  char * provided;
  struct arglist rt;
  struct arglist * a;
+ 
+ 
+ bzero(&rt, sizeof(rt));
 
  if(!fdata){
    rt.type = PKT_ERROR(ERR_FUNC);
    return(rt);
    }
- func 	= harg_get_ptr(functions, harg_get_string(fdata, "atom"));
+ func      = (callback_func_t)harg_get_ptr(functions, harg_get_string(fdata, "atom"));
  arguments = harg_get_ptr(fdata, "args");
- provided = harg_get_string(fdata, "provided");
-
- bzero(&rt, sizeof(struct arglist));
+ provided  = harg_get_string(fdata, "provided");
+ delete_us = emalloc(sizeof(struct arglist));
  if(!func)
  {
   rt = function_call_user(globals, fdata);
   if(rt.type < 0)
   {
-   if(!(rt.type & PKT_ERROR(ERR_EXIT)))
+   if(!(-(rt.type) & ERR_EXIT))
    {
-  printf("Error ! Function \"%s\" does not exist\n", (char*)
-  					harg_get_string(fdata, "atom"));
-  rt.type = PKT_ERROR(ERR_FUNC|ERR_EXIST);
+     fprintf(stderr, "%s - Error ! Function \"%s\" does not exist\n", 
+     					(char*)harg_get_string(globals,"script_name"),
+     					(char*)harg_get_string(fdata, "atom"));
+     rt.type = PKT_ERROR(ERR_FUNC|ERR_EXIST);
    }
-  return(rt);
+   goto cleanup;
   }
-  else return rt;
+  else goto cleanup;
  }
-
+ 
+ 
 if(harg_get_type(fdata, "repeat")>0)
  {
   char * rp = harg_get_string(fdata, "repeat");
@@ -321,9 +343,9 @@ if(harg_get_type(fdata, "repeat")>0)
   rt = sanitize_variable(globals, rp);
   if(rt.type & VAR_STR)repeat = atoi(rt.value);
   else {
-   printf("Error : %s is not a good repeat variable\n", rp);
+   fprintf(stderr, "Error : %s is not a good repeat variable\n", rp);
    rt.type = PKT_ERROR(ERR_VAR|ERR_TYPE);
-   return(rt);
+   goto cleanup;
    }
   if(rt.type & VAR_DELETE)nasl_free(globals,rt.value);
  }else repeat = 1;
@@ -331,6 +353,7 @@ if(harg_get_type(fdata, "repeat")>0)
  /*
   * sanitize the arguments
   */
+ 
  a = arguments;
  while(a && a->next)
  { 
@@ -338,50 +361,85 @@ if(harg_get_type(fdata, "repeat")>0)
   {
   rt = sanitize_variable(globals, a->value);
   if(rt.type){
-    	if(a->value != rt.value)nasl_free(globals, a->value);
-	/*if(rt.type & VAR_STR)*/
-	a->value =  rt.value;
-	a->length = rt.length;
-	/*if(rt.type & VAR_DELETE)nasl_free(globals, rt.value);*/
-	}
+  #if 0
+        a->value = rt.value;
+        a->length = rt.length;
+ #else
+        /*printf("a->value : %s\n", a->value);*/
+        
+        if(a->value && !(a->type & VAR_INT))arg_add_value(delete_us, "delme", ARG_INT, a->length, a->value);
+        if(rt.type & VAR_INT)
+         {
+         a->value = rt.value;
+         a->type = rt.type;
+         if(rt.type & VAR_DELETE)a->type -= VAR_DELETE;
+         a->length = rt.length;
+         }
+        else
+         {
+         a->value =  nstrdup(globals, rt.value,rt.length, 0);
+         a->type = rt.type;
+         if(rt.type & VAR_DELETE)a->type -= VAR_DELETE;
+         a->length = rt.length;
+         }
+        if(rt.type & VAR_DELETE)nasl_free(globals, rt.value);
+#endif
+        }
   }
   a = a->next;
  }
-
+ 
  while(repeat--)
  {
   int err;
   err = evaluate_boolean(globals, provided);
   if(err < 0){
-  	rt.type = err;
-	return(rt);
-	}
+        rt.type = err;
+        goto cleanup;
+        }
   if(err)rt = (*func)(globals, arguments);
  }
 
- 
- /*nasl_arg_free(arguments);*/
- /*nasl_arg_free(fdata);*/
+
+  
+
+cleanup :
+  
+  
+  if(!rt.type) 
+  {
+   char * zero = "0"; 
+   rt.type = VAR_STR|STR_ALL_DIGIT|VAR_DELETE;
+   rt.value = nasl_strdup(globals,zero);
+   rt.length = 1;
+  }
+  
+  
+  
+  while(delete_us && delete_us->next)
+  {
+   struct arglist * next = delete_us->next;
+   free(delete_us->name);
+   nasl_free(globals, delete_us->value);
+   free(delete_us);
+   delete_us = next;
+  }
+  if(delete_us)free(delete_us);
   while(arguments && arguments->next)
   {
-   struct arglist * next;
-   next = arguments->next;
-   free(arguments->name);
-   /*nasl_free(globals, arguments->value);*/
-   free(arguments);
+   struct arglist * next =  arguments->next;
+   nasl_free(globals, arguments->name);
+   if(!(arguments->type & VAR_INT))nasl_free(globals, arguments->value);
+   nasl_free(globals, arguments);
    arguments = next;
   }
-  free(arguments);
+  if(arguments)nasl_free(globals, arguments);
   
   harg_remove(fdata, "args");
   harg_close_all(fdata);
- if(!rt.type) 
- {
-  char * zero = "0"; 
-  rt.type = VAR_STR|STR_ALL_DIGIT;
-  rt.value = nasl_strdup(globals,zero);
-  rt.length = 1;
- }
+ 
+  
+  
  return(rt);
 }
  
