@@ -94,10 +94,10 @@ forge_icmp_packet(globals, args)
    {
     ip_icmp->ip_len = FIX(ip->ip_hl*4 + 8 + len);
     ip_icmp->ip_sum = 0;
-    ip_icmp->ip_sum = np_in_cksum((short*)ip_icmp, ip->ip_hl*4);
+    ip_icmp->ip_sum = np_in_cksum((u_short*)ip_icmp, ip->ip_hl*4);
    }
   }
-  p = pkt + (ip->ip_hl*4);
+  p = (char*)(pkt + (ip->ip_hl*4));
   icmp = (struct icmp*)p;
   
   if((arg_get_type(args, "icmp_code"))>=0)
@@ -123,7 +123,7 @@ forge_icmp_packet(globals, args)
   if(data)bcopy(data, &(p[8]), len);
   
  
-  icmp->icmp_cksum = np_in_cksum((short *) icmp, len+8);
+  icmp->icmp_cksum = np_in_cksum((u_short *) icmp, len+8);
   ret.type = VAR_PKT|PKT_IP|PKT_ICMP;
   ret.value = pkt;
   ret.length = UNFIX(ip->ip_len)+len+8;
