@@ -37,7 +37,7 @@ u_char * copy_variable(globals, var)
 {
  u_char * ret = NULL;
  if(var.type & VAR_INT)ret = var.value;
- else if(var.type & VAR_STR)ret = nstrdup(globals,var.value, var.length, 0);
+ else if(var.type & VAR_STR)ret = (u_char*)nstrdup(globals,var.value, var.length, 0);
  else if(var.type & VAR_ARRAY){
   struct arglist * src = (struct arglist*)var.value;
   harglst * dst = harg_dup((harglst*)src, 0);
@@ -47,7 +47,6 @@ u_char * copy_variable(globals, var)
  {
   ret = nasl_malloc(globals, var.length);
   memcpy(ret, var.value, var.length);
-  /*ret = var.value;*/
  }
 return(ret);
 }
@@ -69,6 +68,7 @@ affect_array_value(globals, var, output)
  u_char * copy;
  int type;
  int len;
+ int code;
  
  /*
   * we have a string like 'xxxx[n] = value'
@@ -76,18 +76,27 @@ affect_array_value(globals, var, output)
  t = nasl_strdup(globals, output);
  array = t;
  index = strchr(t, ARRAY_OPEN);
+ t2 = my_strchr(index, ARRAY_OPEN, ARRAY_CLOSE);
+ 
+ 
  index[0]='\0';
  index++;
- t2 = strchr(index, ARRAY_CLOSE);
+ if(!t2)
+ {
+  fprintf(stderr, "Syntax error - %s\n", output);
+  return PKT_ERROR(ERR_PARSE);
+ }
  t2[0]='\0';
 
  /*
   * index = 'xxx'
   */
  rt = sanitize_variable(globals, index);
- if(rt.value<0)
+ code = (int)rt.value;
+ if(code<0)
  {
-  printf("Error ! Non-existing array !\n");
+  fprintf(stderr, "Error ! Non-existing array !\n");
+  if(rt.type & VAR_DELETE)nasl_free(globals, rt.value);
   return(PKT_ERROR(ERR_VAR|ERR_EXIST));
  }
  
@@ -98,7 +107,9 @@ affect_array_value(globals, var, output)
   {
    if((type != HARG_HARGLST)&&(vtype != VAR_STR))
    {
-    printf("Error ! %s was first declared as a scalar\n", array);
+    fprintf(stderr, "%s - Error ! %s was first declared as a scalar\n",
+    		harg_get_string(globals, "script_name"), array);
+    if(rt.type & VAR_DELETE)nasl_free(globals, rt.value);
     return(PKT_ERROR(ERR_VAR|ERR_ISSCALAR));
    }
   a = harg_get_harg(vars, array);
@@ -119,14 +130,15 @@ affect_array_value(globals, var, output)
   int index = 0;
   if(!(rt.type & STR_ALL_DIGIT))
   {
-   printf("Error ! %s is not a good index for a string\n", (char*)rt.value);
+   fprintf(stderr, "Error ! %s is not a good index for a string\n", (char*)rt.value);
+   if(rt.type & VAR_DELETE)nasl_free(globals, rt.value);
    return(PKT_ERROR(ERR_VAR|ERR_VALUE));
   }
   index = atoi(rt.value);
   if(index < len)
   	s[atoi(rt.value)]=copy[0];
   else {
-  	printf("Warning ! Trying to put data in a too small string\n");
+  	fprintf(stderr, "Warning ! Trying to put data in a too small string\n");
 	}
  }
  else
@@ -134,16 +146,18 @@ affect_array_value(globals, var, output)
   /*
    * Affect the value
    */
- if(harg_get_type(a, rt.value)>0)
+  
+ if(rt.value && (harg_get_type(a, rt.value)>0))
   {
   char * old = harg_get_string(a, rt.value);
   if(!(var.type & VAR_INT) && old)nasl_free(globals, old);
+  
   if(var.type & VAR_INT)harg_set_int(a, rt.value, copy);
   else harg_set_nstring(a, rt.value, var.length, copy);
   }
  else
   {
-   if(var.type & VAR_INT)harg_add_int(a, var.value, copy);
+   if(var.type & VAR_INT)harg_add_int(a, rt.value, copy);
    else harg_add_nstring(a, rt.value, var.length, copy); 
   }
  
@@ -191,21 +205,22 @@ affect_var(globals, var, output)
 { 
  harglst * vars = harg_get_harg(globals, "variables");
  harglst * types= harg_get_harg(globals, "variables_types");
- u_char * copy;
  char * old = NULL;
- var.type = var.type - (var.type & VAR_DELETE);
- copy = copy_variable(globals, var);
+ int delme = 0;
  
- if(var.type & VAR_STR)
+ delme = var.type & VAR_DELETE;
+ var.type = var.type - (var.type & VAR_DELETE);
+
+ if(var.type & (VAR_STR|VAR_PKT))
  {
- old = harg_get_string(vars, output);
- if(old)harg_set_nstring(vars, output, var.length, copy);
+ if((old = harg_get_string(vars, output)))
+   harg_set_nstring(vars, output, var.length, var.value);
  }
  else if(var.type & VAR_INT)
  {
   int iold = harg_get_int(vars, output);
   if(iold > 0){
-  	harg_set_int(vars, output, (int)copy);
+  	harg_set_int(vars, output, (int)var.value);
 	old = (char*)1;
 	}
  }
@@ -213,17 +228,18 @@ affect_var(globals, var, output)
  
  if(!old)
   {
-  if(!copy){
-   copy = nasl_malloc(globals, 2);
-   copy[0]='0';
+  if(!var.value){
+   delme++;
+   var.value = nasl_strdup(globals, "0");
    var.type = VAR_STR;
    var.length = 1;
    }
-  if(var.type & VAR_INT){
-  	harg_add_int(vars, output, copy);
-	}
-  else  harg_add_nstring(vars, output, var.length, copy);
+  if(var.type & VAR_INT)harg_add_int(vars, output, (int)var.value);
+  else  harg_add_nstring(vars, output, var.length, var.value);
  }
+ 
+ if(delme)nasl_free(globals, var.value);
+ 
  if(harg_get_int(types, output)>0)
        harg_set_int(types, output, var.type);
    else
@@ -249,13 +265,13 @@ execute_var_affectation(globals, args)
 
  if(alldigit(output, strlen(output)))
  {
-  printf("%s : Error. %s is not an lvalue\n", instruction, output);
+  fprintf(stderr, "%s : Error. %s is not an lvalue\n", instruction, output);
   return(PKT_ERROR(ERR_VAR|ERR_LVALUE));
  }
  
  if(VAR_CONST & (int)harg_get_int(types, output))
  {
-  printf("%s : Error. %s is a constant\n", instruction, output);
+  fprintf(stderr, "%s : Error. %s is a constant\n", instruction, output);
   return(PKT_ERROR(ERR_VAR|ERR_CONST));
  }
  
@@ -275,14 +291,20 @@ execute_var_affectation(globals, args)
   val = nasl_strdup(globals, harg_get_string(a, "atom"));
   len = harg_get_size(a, "atom")-2;
   if(val[len]==END_INSTRUCTION)val[len]='\0';
-  /*if(quoted_strchr(val, START_FUNCTION))
-   var = execute_func(globals, val);
-  else*/
-   var = sanitize_variable(globals, val);
-  if(var.type < 0)return(var.type);
+  var = sanitize_variable(globals, val);
+  if(var.type < 0){
+  	nasl_free(globals, val);
+  	harg_close_all(a);
+  	return(var.type);
+	}
   if(strchr(output, ARRAY_OPEN))err = affect_array_value(globals, var, output);
   else err = affect_var(globals, var, output);
-  if(err < 0)return(err);
+  if(err < 0){
+  	nasl_free(globals, val);
+  	harg_close_all(a);
+	if(var.type & VAR_DELETE)nasl_free(globals, var.value);
+  	return(err);
+	}
   if(var.type & VAR_DELETE)nasl_free(globals, var.value);
   nasl_free(globals, val);
  }
@@ -297,10 +319,17 @@ execute_var_affectation(globals, args)
   c = x[0];
   x[0] = '\0';
   var = sanitize_variable(globals, instruction);
-  if(var.type < 0)return(var.type);
+  if(var.type < 0){
+  		harg_close_all(a);
+  		return(var.type);
+		}
   /*.... affect here, depending on the type */
   err = affect_var(globals, var, output);
-  if(err < 0)return(err);
+  if(var.type & VAR_DELETE)nasl_free(globals, var.value);
+  if(err < 0){
+  	harg_close_all(a);
+  	return(err);
+	}
   x[0]=c;
  }
  harg_close_all(a);
@@ -410,7 +439,11 @@ execute_single_atom(globals, args)
  char * func = harg_get_string(args, "atom");
  struct arglist ret;
  bzero(&ret, sizeof(ret));
- if(quoted_strchr(func, START_FUNCTION))ret = function_call(globals, func);
+ if(quoted_strchr(func, START_FUNCTION))
+ {
+  ret = function_call(globals, func);
+  if((ret.type > 0) && ret.value)nasl_free(globals, ret.value);
+ }
  return(ret.type<0 ? (int)ret.type:0);
 }
  
@@ -426,13 +459,23 @@ execute_instruction(globals, inst)
   harglst * args;
   int err = 0;
 
+  
   inst = nasl_strdup(globals, inst);
 
-  if((inst[0]==END_INSTRUCTION)&&(inst[1]=='\0'))return(SINGLE_ATOM);
+  if((inst[0]==END_INSTRUCTION)&&(inst[1]=='\0'))
+  {
+   nasl_free(globals, inst);
+   return(SINGLE_ATOM);
+  }
   args = parse_instruction(globals, inst);
+  
   if(!args)
+    {
+    nasl_free(globals, inst);
     return PKT_ERROR(NO_ERR);
+    }
   type = (int)harg_get_int(args, "type");
+  
   switch(type)
   {
    case VAR_AFFECTATION :
