@@ -27,9 +27,15 @@
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
+#else
+extern int getopt (); /* not using prototypes for compat reasons -- jh */
+extern char *optarg;
+extern int optind, opterr, optopt;
 #endif
 
+#if 0 /* not using prototypes for compat reasons -- jh */
 extern int getopt(int, char* const [], const char*);
+#endif
 
 #ifndef MAP_FAILED
 #define MAP_FAILED ((void*)-1)
@@ -60,7 +66,8 @@ init_hostinfos(globals, hostname, ip)
 }
 
 void
-sighandler()
+sighandler(s)
+ int s;
 {
  nasl_exit(Globals);
  exit(0);
@@ -148,6 +155,7 @@ int main(int argc, char ** argv)
  }
  signal(SIGINT, sighandler);
  signal(SIGTERM, sighandler);
+ signal(SIGPIPE, SIG_IGN);
  
  if(!target)target = default_target;
  
