@@ -70,6 +70,7 @@ struct arglist nasl_scanner_add_port(harglst*, struct arglist *);
 struct arglist nasl_scanner_status(harglst*, struct arglist *);
 struct arglist security_hole(harglst*, struct arglist *);
 struct arglist security_warning(harglst*, struct arglist *);
+struct arglist security_note(harglst*, struct arglist *);
 
 struct arglist pkt_recv(harglst*, struct arglist *);
 struct arglist pkt_send(harglst*, struct arglist *);
@@ -92,7 +93,9 @@ struct arglist  script_timeout(globals, vars)
  rt = sanitize_variable(globals, vars->value);
  if(rt.type)
  {
-  plug_set_timeout(script_infos, atoi(rt.value));
+  int to = atoi(rt.value);
+  plug_set_timeout(script_infos, to?to:-1);
+  if(rt.type & VAR_DELETE)nasl_free(globals, rt.value);
  }
  bzero(&rt, sizeof(rt));
  return rt;
@@ -111,6 +114,7 @@ struct arglist  script_id(globals, vars)
  if(rt.type)
  {
   plug_set_id(script_infos, atoi(rt.value));
+  if(rt.type & VAR_DELETE)nasl_free(globals, rt.value);
  }
  bzero(&rt, sizeof(rt));
  return rt;
@@ -128,6 +132,7 @@ struct arglist  script_cve_id(globals, vars)
  if(rt.type)
  {
   plug_set_cve_id(script_infos, rt.value);
+  if(rt.type & VAR_DELETE)nasl_free(globals, rt.value);
  }
  bzero(&rt, sizeof(rt));
  return rt;
@@ -164,7 +169,7 @@ static struct arglist script_elem(globals, vars, name)
  	str = arg_get_value(vars, dfl);
  	if(!str){
 		str = vars->value;
- 		if(!str)printf("ERROR ! NULL %s\n", name);
+ 		if(!str)fprintf(stderr, "ERROR ! NULL %s\n", name);
 		}
 	    }
  if(str)arg_add_value(script_infos, name, ARG_STRING, strlen(str),strdup(str));
@@ -209,8 +214,8 @@ struct arglist script_category(globals, vars)
  struct arglist sa;
  
  if(!vars->value){
- 	printf("Argument error in function script_category()\n");
-	printf("Function usage is : script_category(<category>)\n");
+ 	fprintf(stderr, "Argument error in function script_category()\n");
+	fprintf(stderr, "Function usage is : script_category(<category>)\n");
  	rt.type = PKT_ERROR(ERR_FUNC|ERR_ARG);
 	return(rt);
  	}
@@ -237,9 +242,9 @@ struct arglist  script_dependencie(globals, vars)
  struct arglist * script_infos = harg_get_ptr(globals, "script_infos");
  struct arglist sa;
  if(!vars->value){
- 	printf("Argument error in function script_dependencie()\n");
-	printf("Function usage is : script_dependencie(<name>)\n");
-	printf("Where <name> is the name of another script\n");
+ 	fprintf(stderr, "Argument error in function script_dependencie()\n");
+	fprintf(stderr, "Function usage is : script_dependencie(<name>)\n");
+	fprintf(stderr, "Where <name> is the name of another script\n");
 	rt.type = PKT_ERROR(ERR_FUNC|ERR_ARG);
 	return(rt);
 	}
@@ -265,9 +270,9 @@ struct arglist  script_require_keys(globals, vars)
  struct arglist sa;
 
  if(!vars->value){
- 	printf("Argument error in function script_require_keys()\n");
-	printf("Function usage is : script_require_keys(<name>)\n");
-	printf("Where <name> is the name of a key\n");
+ 	fprintf(stderr, "Argument error in function script_require_keys()\n");
+	fprintf(stderr, "Function usage is : script_require_keys(<name>)\n");
+	fprintf(stderr, "Where <name> is the name of a key\n");
 	rt.type = PKT_ERROR(ERR_FUNC|ERR_ARG);
 	return(rt);
 	}
@@ -293,9 +298,9 @@ struct arglist  script_exclude_keys(globals, vars)
  struct arglist * script_infos = harg_get_ptr(globals, "script_infos");
  struct arglist sa;
  if(!vars->value){
- 	printf("Argument error in function script_exclude_keys()\n");
-	printf("Function usage is : script_exclude_keys(<name>)\n");
-	printf("Where <name> is the name of a key\n");
+ 	fprintf(stderr, "Argument error in function script_exclude_keys()\n");
+	fprintf(stderr, "Function usage is : script_exclude_keys(<name>)\n");
+	fprintf(stderr, "Where <name> is the name of a key\n");
 	rt.type = PKT_ERROR(ERR_FUNC|ERR_ARG);
 	return(rt);
 	}
@@ -323,9 +328,9 @@ struct arglist  script_require_ports(globals, vars)
  struct arglist sa;
  
  if(!vars->value){
- 	printf("Argument error in function script_require_ports()\n");
-	printf("Function usage is : script_require_ports(<name>)\n");
-	printf("Where <name> is the number of a port\n");
+ 	fprintf(stderr, "Argument error in function script_require_ports()\n");
+	fprintf(stderr, "Function usage is : script_require_ports(<name>)\n");
+	fprintf(stderr, "Where <name> is the number of a port\n");
 	rt.type = PKT_ERROR(ERR_FUNC|ERR_ARG);
 	return(rt);
 	}
@@ -354,9 +359,9 @@ struct arglist  script_require_udp_ports(globals, vars)
  struct arglist sa;
  
  if(!vars->value){
- 	printf("Argument error in function script_require_udp_ports()\n");
-	printf("Function usage is : script_require_ports(<name>)\n");
-	printf("Where <name> is the number of a port\n");
+ 	fprintf(stderr, "Argument error in function script_require_udp_ports()\n");
+	fprintf(stderr, "Function usage is : script_require_ports(<name>)\n");
+	fprintf(stderr, "Where <name> is the number of a port\n");
 	rt.type = PKT_ERROR(ERR_FUNC|ERR_ARG);
 	return(rt);
 	}
@@ -384,7 +389,7 @@ struct arglist  script_add_preference(globals, args)
  struct arglist *  script_infos = harg_get_ptr(globals, "script_infos");
  if(!name || !type || !value)
  {
-  printf("Argument error in the call to script_add_preference()\n");
+  fprintf(stderr, "Argument error in the call to script_add_preference()\n");
   rt.type = PKT_ERROR(ERR_FUNC|ERR_ARG);
   return(rt);
  }
@@ -403,23 +408,28 @@ struct arglist script_get_preference(globals, args)
  struct arglist ar;
  rt.type = -1;
  if(!args->value){
- 	printf("Argument error in the function script_get_preference()\n");
-	printf("Function usage is : pref = script_get_preference(<name>)\n");
+ 	fprintf(stderr, "Argument error in the function script_get_preference()\n");
+	fprintf(stderr, "Function usage is : pref = script_get_preference(<name>)\n");
 	rt.type = PKT_ERROR(ERR_FUNC|ERR_ARG);
  	return(rt);
 	}
  ar = sanitize_variable(globals, args->value);
  if(!ar.type)
  {
-  printf("Error in the script\n");
+  fprintf(stderr, "Error in script_get_preference()\n");
   rt.type = PKT_ERROR(ERR_FUNC|ERR_ARG);
   return(rt);
  }
  value = get_plugin_preference(script_infos, ar.value);
  if(ar.type & VAR_DELETE)nasl_free(globals, ar.value);
- rt.value = value;
- rt.length = ar.length;
- rt.type = VAR_STR|VAR_CONST;
+ if(value)
+ {
+ rt.value = nasl_strdup(globals, value);
+ rt.length = strlen(value);
+ rt.type = VAR_STR|VAR_DELETE;
+ }
+ else
+   bzero(&rt, sizeof(rt));
  return(rt);
 }
 
@@ -515,6 +525,7 @@ get_port_state(globals, vars)
 #endif   
    rt.value = value;
    rt.length = 1;
+   if(sa.type & VAR_DELETE)nasl_free(globals, sa.value);
   }
  }
  return(rt);
@@ -550,6 +561,7 @@ get_udp_port_state(globals, vars)
 #endif   
    rt.value = value;
    rt.length = 1;
+   if(sa.type & VAR_DELETE)nasl_free(globals, sa.value);
   }
  }
  return(rt);
@@ -560,8 +572,12 @@ pkt_close(globals, args)
  harglst * globals;
  struct arglist * args;
 {
+ harglst * types = harg_get_harg(globals, "variables_types");
+ harglst * udp_sockets = harg_get_harg(types, "__udp_sockets");
  struct arglist rt;
  struct arglist sa;
+ char * soc_asc;
+ 
  rt.type = 0;
  rt.value = 0;
  
@@ -572,6 +588,10 @@ pkt_close(globals, args)
  	close((int)sa.value);
 	}
  else if(sa.type & VAR_DELETE)nasl_free(globals, sa.value);
+ soc_asc = nasl_malloc(globals,8);
+ sprintf(soc_asc, "%d", (int)sa.value);
+ harg_remove(udp_sockets, soc_asc);
+ nasl_free(globals, soc_asc);
  return(rt);
 }
 
@@ -654,7 +674,7 @@ tryagain :
   * it ready for reuse
   */
  e = 1;
- setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &e, sizeof(int));
+ setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (void*)&e, sizeof(int));
  
  /*
   * Connect to the other end
@@ -725,12 +745,16 @@ pkt_open_sock_tcp(globals, vars)
   rt.value = (void*)soc;
   rt.type  = VAR_INT;
   rt.length = sizeof(int);
+  
+   if(sa.type & VAR_DELETE)nasl_free(globals, sa.value);
+   
+   
   if(soc < 0){	
   		rt.value = 0;
   		rt.type = 0;
 		rt.length = 0;
 		return rt;
-		}
+		}		
   }
  }
  return(rt);
@@ -781,6 +805,7 @@ pkt_open_sock_udp(globals, vars)
    rt.type = VAR_INT;
    rt.value = (void*)soc;
    rt.length = sizeof(int);
+   if(sa.type & VAR_DELETE)nasl_free(globals, sa.value);
    return(rt);
   }
  }
@@ -801,8 +826,8 @@ pkt_ftp_log_in(globals, args)
  bzero(&rt, sizeof(rt));
  soc = (int)arg_get_value(args, "socket");
  if(!soc){
- 	printf("Argument error in function ftp_log_in()\n");
-	printf("Function usage is : result = ftp_log_in(socket:<soc>,user:<user>, pass:<pass>\n");
+ 	fprintf(stderr, "Argument error in function ftp_log_in()\n");
+	fprintf(stderr, "Function usage is : result = ftp_log_in(socket:<soc>,user:<user>, pass:<pass>\n");
  	rt.type = PKT_ERROR(ERR_FUNC|ERR_ARG);
 	return(rt);
 	}
@@ -857,22 +882,25 @@ pkt_is_cgi_installed(globals, args)
  {
   struct arglist ar;
   char * value;
-  
+  int port;
   ar = sanitize_variable(globals, cgi);
   if(ar.type & VAR_STR)
   {
-   value = nasl_malloc(globals,2);
+   value = nasl_malloc(globals,12);
 #ifdef DEBUG_NASL
    printf("is %s installed ? (y or n)\n", ar.value);
    value[0]= (getc(stdin)=='y');
 #else
-   if(is_cgi_installed(script_infos, ar.value))value[0]='1';
-   else value[0]='0';
+   if((port = is_cgi_installed(script_infos, ar.value)))
+    sprintf(value, "%d", port);
+   else 
+    value[0] = '0';
 #endif
-   rt.value = value;
-   rt.length = 1;
-   rt.type = VAR_STR|VAR_CONST;
+   rt.value = nasl_strdup(globals, value);
+   rt.length = strlen(value);
+   rt.type = VAR_STR|VAR_DELETE;
    }
+   if(ar.type & VAR_DELETE)nasl_free(globals, ar.value); 
  }
  return(rt);
 }
@@ -959,22 +987,22 @@ struct arglist security_hole(globals, vars)
  
  
  if(!vars->value){
- 	printf("Syntax error in function security_hole()\n");
-	printf("Usage is : ");
-	printf("\tsecurity_hole(<port>)\n");
-	printf("or\n");
-	printf("\tsecurity_hole(port:<port>, data:<data>, [,proto:<proto>])");
+ 	fprintf(stderr, "Syntax error in function security_hole()\n");
+	fprintf(stderr, "Usage is : ");
+	fprintf(stderr, "\tsecurity_hole(<port>)\n");
+	fprintf(stderr, "or\n");
+	fprintf(stderr, "\tsecurity_hole(port:<port>, data:<data>, [,proto:<proto>])");
 	rt.type = PKT_ERROR(ERR_FUNC|ERR_ARG);
 	return(rt);
        }
  asc_port = vars->value;
  if(proto == asc_port)asc_port = vars->next->value;
  if(!asc_port){
-       printf("Syntax error in function security_hole()\n");
-       printf("Usage is : ");
-       printf("\tsecurity_hole(<port>)\n");
-       printf("or\n");
-       printf("\tsecurity_hole(port:<port>, data:<data>, [,proto:<proto>])");
+       fprintf(stderr, "Syntax error in function security_hole()\n");
+       fprintf(stderr, "Usage is : ");
+       fprintf(stderr, "\tsecurity_hole(<port>)\n");
+       fprintf(stderr, "or\n");
+       fprintf(stderr, "\tsecurity_hole(port:<port>, data:<data>, [,proto:<proto>])");
        rt.type = PKT_ERROR(ERR_FUNC|ERR_ARG);  
 	return(rt);
 	}
@@ -982,7 +1010,7 @@ struct arglist security_hole(globals, vars)
  sa = sanitize_variable(globals, asc_port);
  if(sa.type & VAR_INT)port = (int)sa.value;
  else if(sa.type & VAR_STR)port = atoi(sa.value);
-
+ if(sa.type & VAR_DELETE)nasl_free(globals, sa.value);
 
  if(!proto)
   post_hole(script_infos, port, arg_get_value(script_infos, "DESCRIPTION"));
@@ -1046,7 +1074,7 @@ struct arglist security_warning(globals, vars)
  sa = sanitize_variable(globals, asc_port);
  if(sa.type & VAR_INT)port = (int)sa.value;
  else if(sa.type & VAR_STR)port = atoi(sa.value);
-
+ if(sa.type & VAR_DELETE)nasl_free(globals, sa.value);
  if(!proto)
   post_info(script_infos, port, arg_get_value(script_infos, "DESCRIPTION"));
  else
@@ -1058,6 +1086,66 @@ struct arglist security_warning(globals, vars)
  return(rt);
 }
 
+struct arglist security_note(globals, vars)
+ harglst* globals;
+ struct arglist  * vars;
+{
+  int port = 0;
+ struct arglist * script_infos = harg_get_ptr(globals, "script_infos");
+ struct arglist rt;
+ struct arglist sa;
+ char * proto = arg_get_value(vars, "prototype");
+ char * data = arg_get_value(vars, "data");
+ char * asc_port = arg_get_value(vars, "port");
+ 
+  if((harg_get_int(globals, "standalone"))==1)
+ {
+  if(data)
+   fprintf(stderr, "%s\n", data);
+  else
+   fprintf(stderr, "Success\n");
+ 
+ }
+ 
+ if(!proto)proto = arg_get_value(vars, "protocol");
+#ifdef DEBUG_NASL
+ printf("in security_note\n"); 
+#endif
+ bzero(&rt, sizeof(rt));
+#ifndef DEBUG_NASL
+
+ if(data)
+ {
+  port = atoi(asc_port);
+
+  if(!proto) post_note(script_infos, port, data);
+  else proto_post_note(script_infos, port, proto, data);
+
+  return(rt);
+ }
+ 
+ 
+ if(!vars->value)return(rt);
+ 
+ asc_port = vars->value;
+ if(proto == asc_port)asc_port = vars->next->value;
+ if(!asc_port)return(rt);
+ 
+ sa = sanitize_variable(globals, asc_port);
+ if(sa.type & VAR_INT)port = (int)sa.value;
+ else if(sa.type & VAR_STR)port = atoi(sa.value);
+ if(sa.type & VAR_DELETE)nasl_free(globals, sa.value);
+ if(!proto)
+  post_note(script_infos, port, arg_get_value(script_infos, "DESCRIPTION"));
+ else
+  {
+  proto_post_note(script_infos, port, proto, 
+  	arg_get_value(script_infos, "DESCRIPTION"));
+  }
+#endif
+ return(rt);
+}
+
 struct arglist pkt_recv(globals, vars)
  harglst * globals;
  struct arglist * vars;
@@ -1074,20 +1162,19 @@ struct arglist pkt_recv(globals, vars)
  fd_set rd;
  struct timeval tv;
  int new_len = 0;
+ 
  bzero(&rt, sizeof(rt));
  if(!a_len || !soc)return(rt);
  asc = nasl_malloc(globals,8);
  sprintf(asc, "%d", soc);
  len = atoi(a_len);
  data = nasl_malloc(globals,len+1);
-#ifdef DEBUG_NASL
- rt.value = prompt(globals, "pkt_recv : ");
-#else
  if(arg_get_length(vars, "socket") != sizeof(int))
  {
+  nasl_free(globals, asc);
   return rt;
  }
- 
+
  tv.tv_sec = harg_get_int(globals, "read_timeout");
  if(!tv.tv_sec)tv.tv_sec = 15;
  if(asc_to)tv.tv_sec = atoi(asc_to);
@@ -1110,10 +1197,20 @@ struct arglist pkt_recv(globals, vars)
   
    if(e <= 0)
    {
+#ifdef ECONNRESET
+    if((e < 0) && (errno == ECONNRESET))
+    {
+	    continue;
+    }
+#endif
+
     if(!new_len)
     {
      bzero(&rt, sizeof(rt));
+     nasl_free(globals, data); 
+     nasl_free(globals, asc);
      rt.type = VAR_STR;
+     rt.value = nasl_strdup(globals, "0");
      return rt;
     }
     else break;
@@ -1124,11 +1221,9 @@ struct arglist pkt_recv(globals, vars)
   else break;
   
   bzero(&tv, sizeof(tv));
-  tv.tv_usec = 100;
+  tv.tv_sec = 1;
+  tv.tv_usec = 0;
  }
- 
- rt.value = (void*)nasl_malloc(globals,new_len+1);
-#endif
  nasl_free(globals, asc);
  if(new_len>0)
  {
@@ -1137,7 +1232,8 @@ struct arglist pkt_recv(globals, vars)
  }
  else {
   rt.length = 0;
-  rt.value = NULL;
+  nasl_free(globals, data);
+  rt.value = nasl_strdup(globals, "0");
   }
  rt.type  = VAR_STR;
  return(rt);
@@ -1249,20 +1345,21 @@ struct arglist telnet_init(globals, vars)
  
  bzero(&rt, sizeof(rt));
  if(!vars->value){
- 	printf("Syntax error in the telnet_init() function\n");
-	printf("Correct syntax is : output = telnet_init(<socket>)\n");
+ 	fprintf(stderr, "Syntax error in the telnet_init() function\n");
+	fprintf(stderr, "Correct syntax is : output = telnet_init(<socket>)\n");
 	rt.type = PKT_ERROR(ERR_VAR|ERR_PARSE);
  	return(rt);
 	}
  sa = sanitize_variable(globals, vars->value);
  if(sa.type & VAR_INT)soc = (int)sa.value;
  else {
- 	printf("Argument error in the telnet_init() function\n");
-	printf("Correct syntax is : output = telnet_init(<socket>)\n");
-	printf("Where <socket> has been created with open_sock()\n");
+ 	fprintf(stderr, "Argument error in the telnet_init() function\n");
+	fprintf(stderr, "Correct syntax is : output = telnet_init(<socket>)\n");
+	fprintf(stderr, "Where <socket> has been created with open_sock()\n");
+	if(sa.type & VAR_DELETE)nasl_free(globals, sa.value);
 	rt.type = PKT_ERROR(ERR_VAR|ERR_TYPE);
 	return(rt);
-	}
+	}	
  iac = 255;
  
  tv.tv_sec = harg_get_int(globals, "read_timeout");
@@ -1349,6 +1446,7 @@ tcp_ping(globals, v)
  char * one = "1", * zero = "0";
  struct arglist rt;
  pcap_t * pcap;
+ 
 #ifdef DEBUG_NASL
  rt.value = prompt(globals, "Is the remote host alive ?\n");
  rt.type = VAR_STR;
@@ -1368,21 +1466,20 @@ tcp_ping(globals, v)
  if(islocalhost(dst))flag++;
  else
  {
-  while((i < 2) && (!flag))
+  while((i < 5) && (!flag))
   {
-   int j;
-    
+   bzero(packet, sizeof(packet));
   /* IP */
-  ip->ip_hl = 5;	ip->ip_off = FIX(0);
-  ip->ip_v = 4;		ip->ip_len = FIX(40);
-  ip->ip_tos = 0;	ip->ip_p = IPPROTO_TCP;
-  ip->ip_id = rand();	ip->ip_ttl = 0x40;
+  ip->ip_hl  = 5;	ip->ip_off = FIX(0);
+  ip->ip_v   = 4;	ip->ip_len = FIX(40);
+  ip->ip_tos = 0;	ip->ip_p   = IPPROTO_TCP;
+  ip->ip_id  = rand();	ip->ip_ttl = 0x40;
   ip->ip_src = src; 	ip->ip_dst = *dst;
-  ip->ip_sum = 0;	ip->ip_sum = np_in_cksum((short *)ip, 20);
+  ip->ip_sum = 0;	ip->ip_sum = np_in_cksum((u_short *)ip, 20);
   
   
   /* TCP */ 
-  tcp->th_sport = htons(80);  tcp->th_flags = TH_ACK;
+  tcp->th_sport = (rand()%64000)+1024;  tcp->th_flags = TH_SYN;
   tcp->th_dport = port ? htons(port):htons(80);  tcp->th_seq = rand();
   tcp->th_ack = rand();	tcp->th_x2  = 0;
   tcp->th_off = 5;	tcp->th_win = 2048;
@@ -1415,9 +1512,8 @@ tcp_ping(globals, v)
  pcap = (pcap_t*)init_ip_pcap(globals, ip->ip_dst, ip->ip_src, filter);
  soca.sin_family = AF_INET;
  soca.sin_addr = ip->ip_dst;
- for(j=0;j<5;j++)
-     sendto(soc, ip, 40, 0, (struct sockaddr *)&soca, sizeof(soca));
- if(recv_ip_packet(globals, pcap, 1000))flag++;	
+ sendto(soc, (const void*)ip, 40, 0, (struct sockaddr *)&soca, sizeof(soca));
+ if(recv_ip_packet(globals, pcap, 0))flag++;	
  i++;
   }
  }
@@ -1492,6 +1588,12 @@ start_denial(globals, args)
  int port = plug_get_host_open_port(script_infos);
  int soc;
  struct arglist rt;
+  
+ struct arglist p;
+ char * v;
+ int alive = 0;
+  
+ bzero(&rt, sizeof(rt));
  if(port)
  {
   soc = open_sock_tcp(script_infos, port);
@@ -1499,9 +1601,19 @@ start_denial(globals, args)
   {
    harg_add_int(globals, "denial_port", port);
    close(soc);
+  
+   return rt;
   }
  }
- bzero(&rt, sizeof(rt));
+
+ p = tcp_ping(globals, args);
+ v = p.value;
+ if(v){
+  	alive = (v[0] == '1');
+  	nasl_free(globals, v);
+       }
+  
+ harg_add_int(globals, "tcp_ping_result",  alive);
  return rt;
 }
 
@@ -1523,7 +1635,15 @@ end_denial(globals, args)
 
  if(!port)
  {
-  return tcp_ping(globals, args);
+  int ping = harg_get_int(globals, "tcp_ping_result");
+  harg_remove(globals, "tcp_ping_result");
+  if(ping) return tcp_ping(globals, args);
+  else {
+  	rt.length = 1;
+	rt.value  = nasl_strdup(globals, "1");
+	rt.type   = VAR_STR;
+	return(rt);
+       }
  }
  else 
  {
@@ -1551,6 +1671,49 @@ end_denial(globals, args)
 
  }
  
+ 
+struct arglist
+ cgibin(globals, args)
+  harglst * globals;
+  struct arglist * args;
+{
+ struct arglist * script_infos = harg_get_ptr(globals, "script_infos");
+ struct arglist rt;
+ struct arglist * prefs = arg_get_value(script_infos, "preferences");
+ char * path = arg_get_value(prefs, "cgi_path");
+ 
+ bzero(&rt, sizeof(rt));
+ rt.type = VAR_STR;
+ if(!path)
+ {
+  rt.value = nasl_strdup(globals, "/cgi-bin");
+  rt.length = strlen(rt.value);
+  return rt;
+ }
+ else
+ {
+  char * t;
+  pid_t pid;
+  while(t = strchr(path, ':'))
+  {
+   if(!(pid = fork()))
+   {
+   t[0]='\0';
+   rt.value = nasl_strdup(globals, path);
+   rt.length = strlen(rt.value);
+   return(rt);
+   }
+   else
+    waitpid(pid, NULL, 0);
+    
+   path = t + 1; 
+  }
+  rt.value = nasl_strdup(globals, path);
+  rt.length = strlen(rt.value);
+  return rt;
+ }
+}
+
  struct arglist 
  nasl_islocalhost(globals, args)
   harglst * globals;
@@ -1569,7 +1732,8 @@ end_denial(globals, args)
 }
   
   
-struct arglist nasl_scanner_get_port(globals, args)
+struct arglist 
+nasl_scanner_get_port(globals, args)
  harglst * globals;
  struct arglist * args;
 {
@@ -1584,9 +1748,9 @@ struct arglist nasl_scanner_get_port(globals, args)
  bzero(&rt, sizeof(rt));
 
  if(!args->value){
- 	printf("Argument error in scanner_get_port()\n");
-	printf("Correct usage is : num = scanner_get_port(<num>)\n");
-	printf("Where <num> should be 0 the first time you call it\n");
+ 	fprintf(stderr, "Argument error in scanner_get_port()\n");
+	fprintf(stderr, "Correct usage is : num = scanner_get_port(<num>)\n");
+	fprintf(stderr, "Where <num> should be 0 the first time you call it\n");
 	rt.type = PKT_ERROR(ERR_VAR|ERR_EXIST);
 	return(rt);
 	}
@@ -1598,7 +1762,7 @@ struct arglist nasl_scanner_get_port(globals, args)
   u_short * ports = arg_get_value(script_infos, "ports");
   char * d = nasl_malloc(globals,10);
   struct arglist ar;
-  
+  if(rt.type & VAR_DELETE)nasl_free(globals, rt.value);
   ar.type = VAR_STR|STR_ALL_DIGIT;
   sprintf(d, "%d", ports[i]);
   ar.length = strlen(d);
@@ -1610,6 +1774,123 @@ struct arglist nasl_scanner_get_port(globals, args)
  return(rt);
 #endif
 }
+
+
+struct arglist 
+http_open_socket(globals, args)
+ harglst * globals;
+ struct arglist * args;
+{
+ return pkt_open_sock_tcp(globals, args);
+}
+
+struct arglist
+http_close_socket(globals, args)
+ harglst * globals;
+ struct arglist * args;
+{ 
+ return pkt_close(globals, args);
+}
+
+
+static struct arglist 
+_http_req(globals, args, keyword)
+ harglst * globals;
+ struct arglist * args;
+ char * keyword;
+{
+ struct arglist ret;
+ char * str;
+ char * item = arg_get_value(args, "item");
+ char * port = arg_get_value(args, "port");
+ char * req, *ver;
+ struct arglist * script_infos = harg_get_ptr(globals, "script_infos");
+ 
+ bzero(&ret, sizeof(ret));
+ if(!item || !port)
+ {
+  fprintf(stderr, "%s - Error : http_* functions have the following syntax :\n",
+  			harg_get_string(globals, "script_name"));
+  fprintf(stderr, "    http_*(port:<port>, item:<item>\n");
+  ret.type = PKT_ERROR(ERR_ARG);
+  return ret;
+ } 
+ 
+ req = nasl_malloc(globals, strlen(port) + 10);
+ sprintf(req, "http/%s", port);
+ 
+ ver = plug_get_key(script_infos, req);
+
+ nasl_free(globals, req);
+ 
+ if((ver && !strcmp(ver, "11")))
+ {
+  char* hostname = (char*)plug_get_hostname(script_infos);
+  str = nasl_malloc(globals, strlen(item) + strlen(hostname) + 1024);
+  sprintf(str, "%s %s HTTP/1.1\r\n\
+Connection: Close\r\n\
+Host: %s\r\n\
+Pragma: no-cache\r\n\
+User-Agent: Nessus/1.0 [en] (X11, U; Nessus)\r\n\
+Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, image/png, */*\r\n\
+Accept-Language: en\r\n\
+Accept-Charset: iso-8859-1,*,utf-8\r\n\r\n",
+		keyword,
+		item,
+		hostname);
+ }
+ else
+ {
+ str = nasl_malloc(globals, strlen(item) + 120);
+ sprintf(str, "%s %s HTTP/1.0\r\n\r\n", keyword, item);
+ }
+ 
+ 
+ ret.value = str;
+ ret.length = strlen(str);
+ ret.type = VAR_STR;
+ return ret;
+}
+/*
+ * Syntax :
+ *
+ * http_get(port:<port>, item:<item>);
+ *
+ */
+struct arglist 
+http_get(globals, args)
+ harglst * globals;
+ struct arglist * args;
+{
+ return _http_req(globals, args, "GET");
+}
+
+/*
+ * Syntax :
+ *
+ * http_head(port:<port>, item:<item>);
+ *
+ */
+struct arglist
+http_head(globals, args)
+ harglst * globals;
+ struct arglist * args;
+{
+ return _http_req(globals, args, "HEAD");
+}
+
+
+/*
+ * Syntax :
+ * http_post(port:<port>, item:<item>)
+ */
+struct arglist
+http_post(globals, args)
+ harglst * globals;
+ struct arglist * args;
+{
+ return _http_req(globals, args, "POST");
+}
   
 
 #endif /* defined(NESSUS_EXTENSIONS) */
