@@ -34,6 +34,7 @@ struct arglist get_kb_item(harglst *, struct arglist *);
 
 struct arglist security_hole(harglst *, struct arglist *);
 struct arglist security_warning(harglst *, struct arglist *);
+struct arglist security_note(harglst *, struct arglist *);
 
 struct arglist pkt_recv(harglst *, struct arglist *);
 struct arglist pkt_recv_line(harglst *, struct arglist *);
@@ -42,6 +43,8 @@ struct arglist pkt_send(harglst *, struct arglist *);
 struct arglist telnet_init(harglst *, struct arglist *);
 struct arglist tcp_ping(harglst *, struct arglist *);
 
+struct arglist cgibin(harglst*, struct arglist *);
+
 struct arglist nasl_scanner_add_port(harglst *, struct arglist *);
 struct arglist nasl_scanner_status(harglst *, struct arglist *);
 struct arglist nasl_scanner_get_port(harglst*, struct arglist *);
@@ -52,4 +55,10 @@ struct arglist end_denial(harglst*, struct arglist*);
 
 struct arglist pkt_open_priv_sock_tcp(harglst*, struct arglist*);
 struct arglist pkt_open_priv_sock_udp(harglst*, struct arglist*);
+
+struct arglist http_open_socket(harglst*, struct arglist*);
+struct arglist http_close_socket(harglst*, struct arglist*);
+struct arglist http_get(harglst*, struct arglist*);
+struct arglist http_head(harglst*, struct arglist*);
+struct arglist http_post(harglst*, struct arglist*);
 #endif
