@@ -42,38 +42,40 @@ remove_whitespaces(globals, inst)
   strncpy(blank, inst, STRLEN_FUNC);
   inst+=STRLEN_FUNC;
   i= STRLEN_FUNC;
-  }
+ }
   
  while(inst[0])
  {
   if(inst[0]==OPEN_QUOTE){
-  	 blank[i++]=inst[0];
-	 inst++;
-	 while(inst[0]!=CLOSE_QUOTE)
-	 {
-	  if(!inst[0])
-	  {
-	   fprintf(stderr, "Parse error - missing \"\n");
-	   return NULL;
-	  }
-	  blank[i++]=inst[0];
-	  inst++;
-	 }
-	 blank[i++]=inst[0];
-	}
-  else if(inst[0]==COMMENT_CHAR)inst = strchr(inst, '\n');	
+         blank[i++]=inst[0];
+         inst++;
+         while(inst[0]!=CLOSE_QUOTE)
+         {
+          if(!inst[0])
+          {
+           fprintf(stderr, "%s : Parse error - missing \"\n",
+	   	harg_get_string(globals, "script_name"));
+           return NULL;
+          }
+          blank[i++]=inst[0];
+          inst++;
+         }
+         blank[i++]=inst[0];
+        }
+  else if(inst[0]==COMMENT_CHAR)inst = strchr(inst, '\n');      
   else if((inst[0]!=' ')&&(inst[0]!='\n')&&(inst[0]!='\t')&&
-  	  (inst[0]!='\r'))
-  	blank[i++] = inst[0];
-	
-  inst++;
+          (inst[0]!='\r'))
+        blank[i++] = inst[0];
+        
+  if(!inst)break;
+  else inst++;
  }
- blank = nstrdup(globals,blank, strlen(blank), 1);
+ /*blank = nstrdup(globals,blank, strlen(blank), 1);*/
  return(blank);
 }
  
  
- 
+
  
 /*
  * Reads an instruction in buf. new_buf then points to the next
@@ -95,6 +97,8 @@ read_buf_instruction(globals, buf, new_buf)
  int len;
  int if_c  = 0;
  
+			
+ 
  *new_buf = NULL;
  if(!buf || !buf[0])return(NULL);
  
@@ -102,23 +106,23 @@ read_buf_instruction(globals, buf, new_buf)
   * Skip the comments
   */
  while(buf && (
- 	(buf[0]==' ')||
-	(buf[0]=='\n')||
-	(buf[0]=='\r')||
-	(buf[0]=='\t')||
-	(buf[0]==COMMENT_CHAR)))
+        (buf[0]==' ')||
+        (buf[0]=='\n')||
+        (buf[0]=='\r')||
+        (buf[0]=='\t')||
+        (buf[0]==COMMENT_CHAR)))
  {
- 	if(buf[0]!=COMMENT_CHAR){
-			buf++;
-			
-			/*
-			 *  If the end of the buffer has been reached 
-			 *   then exit
-			 */
-			if(buf[0]=='\0')return(NULL);
-			}
-	else buf = (char*)strchr(buf, '\n');
-		
+        if(buf[0]!=COMMENT_CHAR){
+                        buf++;
+                        
+                        /*
+                         *  If the end of the buffer has been reached 
+                         *   then exit
+                         */
+                        if(buf[0]=='\0')return(NULL);
+                        }
+        else buf = (char*)strchr(buf, '\n');
+                
  }
  
  if(!buf)return(NULL);
@@ -135,16 +139,16 @@ search_end_block :
  while(buf && buf[0] && (buf[0]!=END_INSTRUCTION)&&(buf[0]!=END_BLOCK))
  {
   if(buf[0]==START_BLOCK)
-  	{
-	buf = my_strchr(buf, START_BLOCK, END_BLOCK);
-	break; /* done */
-	}
+        {
+        buf = my_strchr(buf, START_BLOCK, END_BLOCK);
+        break; /* done */
+        }
   else if(buf[0]==START_FUNCTION)
-  	buf = my_strchr(buf, START_FUNCTION, END_FUNCTION); 
+        buf = my_strchr(buf, START_FUNCTION, END_FUNCTION); 
   else if(buf[0]==OPEN_QUOTE){
-  	buf = strchr(buf+1, CLOSE_QUOTE);
-	if(buf)buf++;
-	}
+        buf = strchr(buf+1, CLOSE_QUOTE);
+        if(buf)buf++;
+        }
   else buf++;
  }
  if(!buf || !buf[0])return(NULL);
@@ -154,27 +158,27 @@ search_end_block :
   * Search if there is an else branch
   */
  if(if_c){
-  	char * c = buf + 1;
-	if_c = 0;
-	while(c && (
- 	(c[0]==' ')||
-	(c[0]=='\n')||
-	(c[0]=='\r')||
-	(c[0]=='\t')||
-	(c[0]==COMMENT_CHAR)))
- 	{
- 	 if(c[0]!=COMMENT_CHAR)c++;
-	 else c = (char*)strchr(c, '\n');	
- 	}
-	if(!strncmp(c, ELSE, STRLEN_ELSE)){
-		buf = c;
-		goto search_end_block;
-		}
-	}
+        char * c = buf + 1;
+        if_c = 0;
+        while(c && (
+        (c[0]==' ')||
+        (c[0]=='\n')||
+        (c[0]=='\r')||
+        (c[0]=='\t')||
+        (c[0]==COMMENT_CHAR)))
+        {
+         if(c[0]!=COMMENT_CHAR)c++;
+         else c = (char*)strchr(c, '\n');       
+        }
+        if(!strncmp(c, ELSE, STRLEN_ELSE)){
+                buf = c;
+                goto search_end_block;
+                }
+        }
  buf++;
  c = buf[0];
  buf[0]='\0';
- ret =nasl_strdup(globals,orig);
+ ret = nasl_strdup(globals,orig);
  buf[0]=c;
  *new_buf = buf;
  return(ret);
@@ -188,15 +192,15 @@ search_end_block :
  * In memory :
  *
  * globals 
- *	user_functions
- *		function_name
- *			args
- *			   name : 0
- *			   name : 0
- *			body [string] -> body of the function
- *			
- *		
- */	
+ *      user_functions
+ *              function_name
+ *                      args
+ *                         name : 0
+ *                         name : 0
+ *                      body [string] -> body of the function
+ *                      
+ *              
+ */     
 harglst *
 parse_user_function(globals, instruction)
  harglst * globals;
@@ -243,8 +247,8 @@ parse_user_function(globals, instruction)
  
  /*
   * function myfunc(arg1,arg2)
-  *		    ^
-  *		    t
+  *                 ^
+  *                 t
   */ 
   
  args = harg_create(40);
@@ -371,21 +375,21 @@ parse_if(globals, instruction)
 search_else : 
  else_block = strstr(v, ELSE);
  if(else_block){
- 	/* be sure that we are after an instruction */
-	else_block--;
-	if((else_block[0]==END_INSTRUCTION)||(else_block[0]==END_BLOCK))
-	{
-	 else_block++;
- 	 else_block[0]='\0';
-	 else_block+=STRLEN_ELSE;
-	 else_block =nasl_strdup(globals,else_block);
-	}
-	else {
-	        v = else_block+2;
-		else_block = NULL;
-		goto search_else;
-	     }
-	}
+        /* be sure that we are after an instruction */
+        else_block--;
+        if((else_block[0]==END_INSTRUCTION)||(else_block[0]==END_BLOCK))
+        {
+         else_block++;
+         else_block[0]='\0';
+         else_block+=STRLEN_ELSE;
+         else_block =nasl_strdup(globals,else_block);
+        }
+        else {
+                v = else_block+2;
+                else_block = NULL;
+                goto search_else;
+             }
+        }
  inst_block =nasl_strdup(globals,t+1);
  t[0]='\0';
  v = (char*)strchr(inst, START_FUNCTION);
@@ -426,7 +430,7 @@ parse_affectation(globals, instruction)
  harglst * globals;
  char * instruction;
 {
- char * inst =nasl_strdup(globals,instruction);
+ char * inst = nasl_strdup(globals,instruction);
  char * v;
  harglst * ret;
 
@@ -478,4 +482,4 @@ parse_instruction(globals, instruction)
  if(instruction[0]==START_BLOCK)return(parse_block(globals, instruction));
  if(quoted_strchr(instruction, AFFECTATION))return(parse_affectation(globals,instruction));
  else return(parse_singleton(globals, instruction));
-}		
+}               
