@@ -25,7 +25,11 @@
 #include "sanitize.h"
 #include "defines.h"
 #include "nasl_memory.h"
-#include "regex.h"
+#ifdef HAVE_REGEX_SUPPORT
+#include <regex.h>
+#else
+#include "nasl_regex.h"
+#endif
 #include "instruction.h"
 
 /*
@@ -52,13 +56,18 @@ nasl_return(globals, args)
 {
   struct arglist rt;
   char * val;
-   
+  int delme = 0;
   val = args->value;
-  if(!val)val = "0";
+  if(!val){
+  	val = nasl_strdup(globals, "0");
+	delme++;
+	}
   rt = sanitize_variable(globals, val);
+  if(delme)nasl_free(globals, val);
   if(rt.value)
   {
     affect_var(globals, rt, "__nasl_return");
+    if(rt.type & VAR_DELETE)nasl_free(globals, rt.value);
     bzero(&rt, sizeof(rt));
     rt.type = PKT_ERROR(ERR_FUNC_EXIT);
     return rt;
@@ -88,6 +97,7 @@ struct arglist nasl_asctime(globals, args)
    rt.value = buf;
    rt.type = VAR_STR;
    rt.length = strlen(buf);
+   if(al.type & VAR_DELETE)nasl_free(globals, al.value);
   }
  }
  return(rt);
@@ -115,11 +125,11 @@ struct arglist shift_left(globals, args)
   if(asc_size)size = atoi(asc_size);
   else size = 8;
   if((size != 8)&&(size!=16)&&(size!=32))
-	printf("shift_left : <size> must be 8,16 or 32\n");
+	fprintf(stderr, "shift_left : <size> must be 8,16 or 32\n");
   if(asc_num && value)				
      num = atoi(asc_num);
   else {
-	 printf("Usage : shift_left(value:<value>, offset:<offset>)\n");
+	 fprintf(stderr, "Usage : shift_left(value:<value>, offset:<offset>)\n");
 	   return(rt);
        }
 			
@@ -184,12 +194,12 @@ struct arglist shift_right(globals, args)
   if(asc_size)size = atoi(asc_size);
   else size = 8;
   if((size != 8)&&(size!=16)&&(size!=32))
-	printf("shift_right : <size> must be 8,16 or 32\n");
+	fprintf(stderr, "shift_right : <size> must be 8,16 or 32\n");
   if(asc_num && value)				
      num = atoi(asc_num);
   else {
-	 printf("Usage : shift_right(value:<value>, offset:<offset>)\n");
-	   return(rt);
+	 fprintf(stderr, "Usage : shift_right(value:<value>, offset:<offset>)\n");
+	 return(rt);
        }
 			
    l_data = (long*)value;
@@ -250,8 +260,10 @@ struct arglist pkt_hex(globals, args)
    rt.length = strlen(ret);
    rt.value = nstrdup(globals,ret, rt.length, 1);
    rt.type = VAR_STR;
+   if(sa.type & VAR_DELETE)nasl_free(globals, sa.value);
    return(rt);
   }
+  if(sa.type & VAR_DELETE)nasl_free(globals, sa.value);
   args = args->next;
  }
  return(rt);
@@ -276,10 +288,10 @@ struct arglist pkt_ord(globals, args)
    rt.type = VAR_STR|STR_ALL_DIGIT;
    rt.length = strlen(ret);
    rt.value = nstrdup(globals,ret, rt.length, 1);
-   return(rt);
   }
+  if(ar.type & VAR_DELETE)nasl_free(globals, ar.value);
  }
- else printf("ord() usage : ord(char)\n");
+ else fprintf(stderr, "ord() usage : ord(char)\n");
  return(rt);
 }
 
@@ -309,10 +321,10 @@ rawtostr(globals, args)
    rt.length = strlen(ret);
    rt.value = nstrdup(globals,ret, rt.length,1);
    rt.type = VAR_STR|STR_ALL_DIGIT;
-   return(rt);
   }
+  if(sa.type & VAR_DELETE)nasl_free(globals, sa.value);
  }
- printf("Usage : rawtostr(<string>)\n");
+ else fprintf(stderr, "Usage : rawtostr(<string>)\n");
  return(rt);
 }
 
@@ -332,14 +344,14 @@ strtoint(globals, args)
  
  if(!number||!size)
  {
-  printf("strtoint() usage : \n");
-  printf("strtoint(number:<number>, size:<size>)\n");
+  fprintf(stderr, "strtoint() usage : \n");
+  fprintf(stderr, "strtoint(number:<number>, size:<size>)\n");
   return(rt);
  }
  num = htonl(atoi(number));
  len = atol(size);
  if(len > sizeof(long)){
- 	printf("strtoint() size must be at max %d\n", sizeof(long));
+ 	fprintf(stderr, "strtoint() size must be at max %d\n", sizeof(long));
 	return(rt);
 	}
 	
@@ -350,7 +362,7 @@ strtoint(globals, args)
   ret[j++]=s[i];
  }
  rt.length = len;
- rt.value = nstrdup(globals,ret, rt.length,1);
+ rt.value = nstrdup(globals,(char*)ret, rt.length,1);
  rt.type = VAR_STR;
  return(rt);
 }
@@ -415,6 +427,8 @@ struct arglist ereg(globals, args)
   struct re_pattern_buffer pb;
   struct re_registers regs;
  
+  bzero(&pb, sizeof(pb));
+  bzero(&regs, sizeof(regs));
   pb.allocated = 8;
   pb.buffer = nasl_malloc(globals, 8);
   pb.fastmap = fastmap;
@@ -458,7 +472,7 @@ struct arglist ereg(globals, args)
  * Ripped from the php3 team !
  */ 
 /* this is the meat and potatoes of regex replacement! */
-static char * _regreplace(const char *pattern, 
+static char * _regreplace(harglst* globals, const char *pattern, 
 		const char *replace, const char *string, int icase, int extended)
 {
 	regex_t re;
@@ -486,7 +500,7 @@ static char * _regreplace(const char *pattern,
 	/* start with a buffer that is twice the size of the stringo
 	   we're doing replacements in */
 	buf_len = 2 * string_len + 1;
-	buf = emalloc(buf_len * sizeof(char));
+	buf = nasl_malloc(globals, buf_len * sizeof(char));
 	if (!buf) {
 		return NULL;
 	}
@@ -525,9 +539,9 @@ static char * _regreplace(const char *pattern,
 
 			if (new_l + 1 > buf_len) {
 				buf_len = 1 + buf_len + 2 * new_l;
-				nbuf = emalloc(buf_len);
+				nbuf = nasl_malloc(globals, buf_len);
 				strcpy(nbuf, buf);
-				efree(&buf);
+				nasl_free(globals, buf);
 				buf = nbuf;
 			}
 			tmp = strlen(buf);
@@ -560,9 +574,9 @@ static char * _regreplace(const char *pattern,
 				new_l = strlen (buf) + 1;
 				if (new_l + 1 > buf_len) {
 					buf_len = 1 + buf_len + 2 * new_l;
-					nbuf = emalloc(buf_len * sizeof(char));
+					nbuf = nasl_malloc(globals, buf_len * sizeof(char));
 					strcpy(nbuf, buf);
-					efree(&buf);
+					nasl_free(globals, buf);
 					buf = nbuf;
 				}
 				pos += subs[0].rm_eo + 1;
@@ -575,9 +589,9 @@ static char * _regreplace(const char *pattern,
 			new_l = strlen(buf) + strlen(&string[pos]);
 			if (new_l + 1 > buf_len) {
 				buf_len = new_l + 1; /* now we know exactly how long it is */
-				nbuf = emalloc(buf_len * sizeof(char));
+				nbuf = nasl_malloc(globals, buf_len * sizeof(char));
 				strcpy(nbuf, buf);
-				efree(&buf);
+				nasl_free(globals, buf);
 				buf = nbuf;
 			}
 			/* stick that last bit of string on our output */
@@ -611,7 +625,7 @@ struct arglist ereg_replace(globals, args)
   return(rt);
  }
 
- r = _regreplace(pattern, replace, string, icase?(icase[0]=='1'):0, 1);
+ r = _regreplace(globals, pattern, replace, string, icase?(icase[0]=='1'):0, 1);
  rt.value  = nasl_strdup(globals, r);
  rt.length = strlen(r);
  rt.type = VAR_STR;
@@ -746,14 +760,14 @@ crap(globals, args)
  int len;
  struct arglist rt;
  char * data = arg_get_value(args, "data");
- 
+ int data_len = arg_get_length(args, "data");
  bzero(&rt, sizeof(rt));
  if(!data)vr = sanitize_variable(globals, args->value);
  else {
  	char * var;
  	var = arg_get_value(args, "length");
 	if(!var){
-		printf("Script error. crap() should have the 'length' argument\n");
+		fprintf(stderr, "Script error. crap() should have the 'length' argument\n");
 		return(rt);
 		}
 	else vr = sanitize_variable(globals, var);
@@ -765,9 +779,10 @@ crap(globals, args)
   if(!data)memset(ret, 'X', len);
   else {
   	int i,t,r;
-	t = len /  strlen(data);
+	if(!data_len)t = len;
+	else t = len / data_len;
 	for(i=0;i<t;i++)strcat(ret, data);
-	if((r = len % strlen(data)))strncat(ret, data, r);
+	if((r = len % data_len))strncat(ret, data, r);
        }
   rt.type = VAR_STR;
   rt.value  = ret;
@@ -848,7 +863,7 @@ raw_string(globals, args)
 	}
 	
   if((total_len + current_len)>2048){
-  		printf("Error. Too long argument in raw_string()\n");
+  		fprintf(stderr, "Error. Too long argument in raw_string()\n");
 		break;
 		}
   bcopy(str, &(ret[total_len]), current_len);
@@ -857,7 +872,7 @@ raw_string(globals, args)
  }
 
  args = args->next;
- /*if(vv.type & VAR_DELETE)free(vv.value);*/
+ if(vv.type & VAR_DELETE)nasl_free(globals, vv.value);
  }
 
  rt.type = VAR_STR|STR_PURIFIED;
@@ -977,7 +992,8 @@ display(globals, args)
  struct arglist rt;
  int i;
  
- for(i=0;i<length;i++)printf("%c", isprint(s[i])?s[i]:s[i]=='\n'?s[i]:'.');
+ if(s)
+  for(i=0;i<length;i++)printf("%c", isprint(s[i])?s[i]:s[i]=='\n'?s[i]:'.');
  rt.value =  NULL;
  rt.type = 0;
  rt.length = 0;
@@ -994,6 +1010,7 @@ pkt_strlen(globals, args)
  struct arglist rt;
  char * t = "0";
  bzero(&rt, sizeof(rt));
+
  if(sa.value && (sa.type & VAR_STR))
  {
   char * s = sa.value;
@@ -1177,3 +1194,57 @@ free_pkt(globals, args)
  bzero(&rt, sizeof(rt));
  return(rt);
 }
+
+struct arglist
+include(globals, args)
+ harglst * globals;
+ struct arglist * args;
+{
+ char * fullname = nasl_strdup(globals, harg_get_string(globals, "script_name"));
+ char * t;
+ struct arglist filename;
+ struct arglist ret;
+
+ bzero(&ret, sizeof(ret));
+ bzero(&filename, sizeof(filename));
+ if(args->value)filename = sanitize_variable(globals, args->value);
+ if(!filename.value)
+ {
+	 fprintf(stderr, "%s : Error in include(). Usage : include(<file>)\n",
+			 fullname);
+	 ret.type =PKT_ERROR(ERR_FUNC|ERR_ARG);
+	 return ret;
+ }
+
+ t = filename.value;
+ if(t[0] == '/')
+ {
+	 fprintf(stderr, "%s : Error in include(). Can not include absolute pathnames\n",harg_get_string(globals, "script_name"));
+	 ret.type = PKT_ERROR(ERR_FUNC|ERR_ARG);
+	 return ret;
+ }
+
+ if(strstr(t, "../"))
+ {
+	 fprintf(stderr, "%s : Error in include() - file name contains dots and slashes\n", harg_get_string(globals, "script_name"));
+	 ret.type = PKT_ERROR(ERR_FUNC|ERR_ARG);
+	 return ret;
+ }
+
+ t = strrchr(fullname, '/');
+ if(t)
+ {
+ t[0] = '\0';
+ t = nasl_malloc(globals, strlen(fullname) + strlen(filename.value) + 2);
+ sprintf(t, "%s/%s", fullname, filename.value);
+ nasl_free(globals, fullname);
+ }
+ else
+  t = nasl_strdup(globals, filename.value);
+
+ execute_script(globals, t);
+
+ nasl_free(globals, t);
+ return ret;
+}
+ 
