@@ -136,6 +136,7 @@ sanitize_variable(globals, variable)
  if(is_function(variable))
  {
   rt = function_call(globals, variable);
+  rt.type |= VAR_DELETE;
   return(rt);
  }
  
@@ -174,11 +175,20 @@ sanitize_variable(globals, variable)
    char* index;
    struct arglist r;
    harglst * ar;
+   
    t = nasl_strdup(globals, variable);
    index = strchr(t, ARRAY_OPEN);
    index[0] = '\0';
    index++;
    t2 = strchr(index, ARRAY_CLOSE);
+   if(!t2)
+   {
+    fprintf(stderr, "** Parse error - %s\n", index);
+    bzero(&rt, sizeof(rt));
+    printf("variable : %s\n", variable);
+    rt.type = PKT_ERROR(ERR_PARSE);
+    return(rt);
+   }
    t2[0]='\0';
    array = t;
    r = sanitize_variable(globals, index);
@@ -190,13 +200,15 @@ sanitize_variable(globals, variable)
     { 
       int s = 1;
       rt.value = harg_get_string(ar, index);
+      rt.type = 0;
       if(!rt.value){
       	rt.value = (void*)harg_get_int(ar, index);
+	rt.type = VAR_INT;
       	s--;
 	}
       rt.length = harg_get_size(ar, index)-s;
       ar = harg_get_harg(types, array);
-      rt.type = harg_get_int(ar, index);
+      if(!rt.type)rt.type = harg_get_int(ar, index);
       nasl_free(globals, t);
       if(r.type & VAR_DELETE)nasl_free(globals, r.value);
       return(rt);
@@ -205,17 +217,31 @@ sanitize_variable(globals, variable)
     {
      	char * s = (char*)ar;
 	char * v;
+	size_t len = harg_get_size(vars, array);
 	v = nasl_malloc(globals,2);
+	if(atoi(index) > len)
+	{
+	 
+	 fprintf(stderr, "Requesting %s[%d] whereas %s's length is %d !\n",
+	 			t,atoi(index),t,len);
+	 bzero(&rt, sizeof(rt));
+	 rt.type = VAR_STR|VAR_DELETE;
+	 rt.value = nasl_strdup(globals, "0");
+	 rt.length = 1;
+	 return(rt);
+	}			
 	v[0]=s[atoi(index)];
 	rt.value = v;
 	rt.type = VAR_STR | VAR_DELETE;
 	rt.length = strlen(v);
+	nasl_free(globals, t);
+	if(r.type & VAR_DELETE)nasl_free(globals, r.value);
 	return(rt);
      }
     }
    if(r.type & VAR_DELETE)nasl_free(globals, r.value);
    nasl_free(globals, t);
-   } 
+   }
    
    /*
     * 3 (ter) : a structure 
@@ -288,41 +314,26 @@ sanitize_variable(globals, variable)
 #define OP_SHIFT_LEFT 9
 #define OP_SHIFT_RIGHT 10
   {
-  int operator = 0;
+  int operator = -1;
   i = 0;
-  while(!operator && variable[i])
-  	{
-	 register char c;
-	 while(isalnum(variable[i]))i++;
-	 c = variable[i];
-	 if(c =='"'){
-	 	char * s = strchr(&(variable[i+1]), '"');
-		if(!s){
-			printf("syntax error.\n");
-			}
-		else i = i+(strlen(&variable[i])-strlen(s));
-		}
-	 if(c == '+')operator=OP_ADD;
-	 else if(c=='-')operator=OP_SUB;
-	 else if(c=='*')operator=OP_MUL;
-	 else if(c=='/')operator=OP_DIV;
-	 else if(c=='&')operator=OP_AND;
-	 else if(c=='|')operator=OP_OR;
-	 else if(c=='^')operator=OP_XOR;
-	 else if(c=='%')operator=OP_MOD;
-	 else if((c=='<') && (variable[i+1]==c)){
-	 	operator=OP_SHIFT_LEFT;
-		variable[i]=0;
-		i++;
-		}
-	 else if((c=='>') && (variable[i+1]==c)){
-	 	operator=OP_SHIFT_RIGHT;
-		variable[i]=0;
-		i++;
-		}
-	 else if(!c)operator = -1;
-	 else i++;
-	}
+  
+  i = search_op(variable, "+-*/&|^%");
+  
+  if(i > 0)
+  {
+   switch(variable[i])
+   {
+    case '+' : operator = OP_ADD; break;
+    case '-' : operator = OP_SUB; break;
+    case '*' : operator = OP_MUL; break;
+    case '/' : operator = OP_DIV; break;
+    case '&' : operator = OP_AND; break;
+    case '|' : operator = OP_OR;  break;
+    case '^' : operator = OP_XOR; break;
+    case '%' : operator = OP_MOD; break;
+   }
+  }
+  
   if(operator>0)
   {
    int value = 0;
@@ -375,9 +386,11 @@ sanitize_variable(globals, variable)
 	  case OP_XOR : r[i] = r[i] ^ t[i];break;
 	 }
 	}
-	rt.type = VAR_STR;
+	rt.type = VAR_STR|VAR_DELETE;
 	rt.length = maxlen;
 	rt.value = r;
+	if(rv.type & VAR_DELETE)nasl_free(globals, rv.value);
+	if(rst.type & VAR_DELETE)nasl_free(globals, rst.value);
 	return(rt);
        }
        break;
@@ -411,7 +424,6 @@ sanitize_variable(globals, variable)
 	 {
 	 rt.value = nstrdup(globals, v, len_v,0);
 	 rt.length = len_v;
-	 rt.type = VAR_STR;
 	 }
 	}break;
      case OP_ADD : {
@@ -421,7 +433,9 @@ sanitize_variable(globals, variable)
 	 rt.length =  rv.length + rst.length;
 	 }break;
      }
-     rt.type = VAR_STR;
+     rt.type = VAR_STR|VAR_DELETE;
+     if(rv.type & VAR_DELETE)nasl_free(globals, rv.value);
+     if(rst.type & VAR_DELETE)nasl_free(globals, rst.value);	
      return(rt);
      }
     else
@@ -432,6 +446,9 @@ sanitize_variable(globals, variable)
       rt.type = PKT_ERROR(ERR_VAR|ERR_TYPE);
      }
      else rt.type = (rst.type < 0) ? rst.type:rv.type;
+     
+    if(rv.type & VAR_DELETE)nasl_free(globals, rv.value);
+    if(rst.type & VAR_DELETE)nasl_free(globals, rst.value); 
     return(rt);
    }
    if(v && rest)
@@ -452,6 +469,11 @@ sanitize_variable(globals, variable)
    sprintf(ret, "%u", (unsigned)value);
    r = nasl_strdup(globals, ret);
    nasl_free(globals, ret);
+   if(rv.type & VAR_DELETE)nasl_free(globals, rv.value);
+   if(rst.type & VAR_DELETE){
+   	nasl_free(globals, rst.value);
+	}
+	
    rt.type = VAR_STR|VAR_DELETE|STR_ALL_DIGIT;
    rt.value = r;
    rt.length = strlen(r);
@@ -486,7 +508,8 @@ sanitize_variable(globals, variable)
   * 8 : failed
   */
  bzero(&rt, sizeof(rt));
- printf("Warning : evaluating unknown variable - %s\n", variable);
+ fprintf(stderr, "%s : Warning : evaluating unknown variable - %s\n",
+ 	harg_get_string(globals, "script_name"), variable);
  rt.type = PKT_ERROR(ERR_VAR|ERR_EXIST);
  rt.length = 0;
  return(rt);
