@@ -19,7 +19,7 @@
  */
 #include <includes.h>
 #include "nasl_memory.h"
-#include "strutils.h"
+/*#include "strutils.h"*/
 #include "defines.h"
 
 /*
@@ -109,7 +109,14 @@ quoted_strchr(str, chr)
   str[len]=chr; /* set an end */
   while(s[0]!=chr)
   {
-   if(s[0]==OPEN_QUOTE)s = strchr(s+1, CLOSE_QUOTE);
+   if(s[0]==OPEN_QUOTE){
+escape : 
+   	s = strchr(s+1, CLOSE_QUOTE);
+	if(s)
+	{ 
+	 if((s-1)[0]=='\\')goto escape;
+	}
+      }
    if(!s)break;
    else s++;
   }
@@ -136,16 +143,23 @@ quoted_parenthesed_strchr(str, chr)
 	str = strchr(str+1, CLOSE_QUOTE);
         if(!str)
 	 return NULL;
-        else str+=sizeof(char);
+        else str++;
       }
       else if(str[0]==START_FUNCTION)
       {
 	 str = strchr(str+1, END_FUNCTION);
 	 if(!str)
 	   return NULL;
-	 else str+=sizeof(char);
+	 else str++;
       }
-      else str+=sizeof(char);
+      else if(str[0]==ARRAY_OPEN)
+      {
+       str = strchr(str+1, ARRAY_CLOSE);
+       if(!str)
+         return NULL;
+      else str++;
+     }
+      else str++;
       if(!str[0])return NULL;
     }
   }
@@ -154,6 +168,49 @@ quoted_parenthesed_strchr(str, chr)
 
 
 
+int
+search_op(str, ops)
+  char * str;
+  char * ops;
+  
+{
+   int index = 0;
+   char * start = str;
+   
+   for(;;)
+    {
+      if(str[0]==OPEN_QUOTE)
+      {
+	str = strchr(str+1, CLOSE_QUOTE);
+        if(!str)
+	 return -1;
+        else str++;
+      }
+      else if(str[0]==START_FUNCTION)
+      {
+	 str = strchr(str+1, END_FUNCTION);
+	 if(!str)
+	   return -1;
+	 else str++;
+      }
+      else if(str[0]==ARRAY_OPEN)
+      {
+       str = strchr(str+1, ARRAY_CLOSE);
+       if(!str)
+         return -1;
+      else str++;
+     }
+      else str++;
+      if(!str[0])return -1;
+      if(strchr(ops, str[0]))break;
+    }
+    
+    index = (int)(str - start);
+    return index;
+}
+
+
+
 char * nstrdup(globals, str, length, do_free)
  harglst * globals;
  char * str;
